<?php

use Dash_Effortless_Digital_Asset_Management_OAuth as OAuth;
use Dash_Effortless_Digital_Asset_Management_Logging as Logger;
use Dash_Effortless_Digital_Asset_Management_Api as Api;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://brightminded.com
 * @since      1.0.0
 *
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/admin
 * @author     Brightminded <brightminded@brightminded.com>
 */
class Dash_Effortless_Digital_Asset_Management_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->api = new Api();

	}

	/**
	 * Adds the Dash  admin authentication page
	 *
	 * @since    1.0.0
	 * @access   public
	 */
	public function dash_admin_page() {

		add_menu_page(
		
			'Dash',
			'Dash',
			'create_users',
			'dash_auth',
			array( $this, 'build_dash_admin_page' ),
			'dashicons-plugins-checked'
		
		);
	
	}

	/**
	 * It retrieves the various OAuth required authorization settings
	 *
	 * @since    1.0.0
	 * @access   public
	 * @return	 array 		A dictionary of OAuth2 authorization settings
	 */
	public static function get_auth_settings() {

		return array(

			DASH_OAUTH_DB_CLIENT_ID_KEY        => get_option( DASH_OAUTH_DB_CLIENT_ID_KEY ) ? get_option( DASH_OAUTH_DB_CLIENT_ID_KEY ) : '',
			DASH_OAUTH_DB_CLIENT_SECRET_KEY    => get_option( DASH_OAUTH_DB_CLIENT_SECRET_KEY ) ? get_option( DASH_OAUTH_DB_CLIENT_SECRET_KEY ) : '',
			DASH_OAUTH_DB_CLIENT_SUBDOMAIN_KEY => get_option( DASH_OAUTH_DB_CLIENT_SUBDOMAIN_KEY ) ? get_option( DASH_OAUTH_DB_CLIENT_SUBDOMAIN_KEY ) : '',
			DASH_OAUTH_REDIRECT_URI_KEY        => DASH_OAUTH_REDIRECT_URI

		);

	}

	/**
	 * It stores the OAuth token data retrieved from the DASH API during the token step.
	 *
	 * @since    1.0.0
	 * @access   public
	 * @param    League\OAuth2\Client\Token\AccessTokenInterface A token object retrieved from the OAuth process.
	 */
	public static function set_auth_token( $tokenObj ) {

		update_option( 
            DASH_AUTHENTICATED_TOKEN_KEY, 

            array(
                'access'  => $tokenObj->getToken(),
                'refresh' => $tokenObj->getRefreshToken(),
                'expires' => $tokenObj->getExpires()  
            )

        );

	}

	/**
	 * The template code for the Dash authenication page
	 *
	 * @since    1.0.0
	 * @access   public
	 */
	public function build_dash_admin_page() {

		$settings = self::get_auth_settings();

		include( 'partials/dash-effortless-digital-asset-management-admin-display.php' );

	}

	/**
	 * The admin_init action which attempts the OAuth2 authorize
	 * step with the DASH API.
	 *
	 * @since    1.0.0
	 * @access   public
	 */
	public function dash_auth() {

		if ( 
			isset( $_POST[ 'dash-auth-save' ], $_POST[ 'action' ], $_POST[ '_wpnonce' ] ) && 
			wp_verify_nonce( $_POST[ '_wpnonce' ], 'dash_auth' ) &&
			$_POST[ 'action' ] === 'dash_auth' &&
			current_user_can( DASH_ROLE_CAPABILITY )
		 ) {

			foreach ( array( DASH_OAUTH_DB_CLIENT_ID_KEY, DASH_OAUTH_DB_CLIENT_SECRET_KEY, DASH_OAUTH_DB_CLIENT_SUBDOMAIN_KEY ) as $prop ) {

				if ( isset( $_POST[ $prop ] ) ) {

					$value = sanitize_text_field( $_POST[ $prop ] );
					update_option( $prop, $value );

				}

			}

			$settings = self::get_auth_settings();

			OAuth::authorize( 
				$settings[ DASH_OAUTH_DB_CLIENT_ID_KEY ],
				$settings[ DASH_OAUTH_DB_CLIENT_SECRET_KEY ],
				$settings[ DASH_OAUTH_REDIRECT_URI_KEY ],
				$settings[ DASH_OAUTH_DB_CLIENT_SUBDOMAIN_KEY ]
			);

		}

	}

	/**
	 * The admin_init action which attempts the OAuth2 token retrieve
	 * step with the DASH API.
	 *
	 * @since    1.0.0
	 * @access   public
	 */
	public function dash_auth_callback( $current_screen ) {

		if ( isset( $current_screen ) && $current_screen->base === 'toplevel_page_dash_auth' ) {

			if ( empty( $_GET[ 'state' ] ) || ( isset( $_SESSION[ 'oauth2state' ] ) && $_GET[ 'state' ] !== $_SESSION[ 'oauth2state' ] ) ) {

			    if ( isset( $_SESSION[ 'oauth2state' ] ) ) {

			        unset( $_SESSION[ 'oauth2state' ] );
			    
			    }

			}

			if( isset( $_GET[ 'code' ] ) ) {

			    $settings = self::get_auth_settings();

			    $tokenObj = OAuth::token( 
			        $settings[ DASH_OAUTH_DB_CLIENT_ID_KEY ],
			        $settings[ DASH_OAUTH_DB_CLIENT_SECRET_KEY ],
			        $settings[ DASH_OAUTH_REDIRECT_URI_KEY ],
			        wp_unslash( sanitize_text_field( $_GET[ 'code' ] ) )
			    );

			    if ( $tokenObj ) {

			        self::set_auth_token( $tokenObj );

			    }

			}

		}

	}

	/**
	 * It attempts to download an asset from DASH by first requesting the download and subsequently
	 * polling for the link. It performs at most DASH_MAX_DOWNLOAD_RETRIES polling request before
	 * stopping at an interval defined in DASH_DOWNLOAD_POLL_INTERVAL_MICROS.
	 * 
	 * @since    1.0.0
	 * @access   public
	 * @return	 array 		The JSON payload with the download link from DASH.
	 */
	public function dash_download_image() {

		if ( isset( 
			$_POST[ 'postdata' ], 
			$_POST[ 'postdata' ][ 'postid' ], 
			$_POST[ 'postdata' ][ 'imageid' ], 
			$_POST[ 'postdata' ][ 'imagefile' ],
			$_POST[ 'postdata' ][ 'imagetransf' ] ) && 
			current_user_can( DASH_ROLE_CAPABILITY, intval( $_POST[ 'postdata' ][ 'postid' ] ) ) ) {

			$link = '';

			$imagefile = sanitize_text_field( $_POST[ 'postdata' ][ 'imagefile' ] );
			$imagetransf = sanitize_text_field( $_POST[ 'postdata' ][ 'imagetransf' ] );
			$imageid = sanitize_text_field( $_POST[ 'postdata' ][ 'imageid' ] );

			$response = $this->api->assetDownload( $imageid, $imagetransf );

			return wp_send_json_success( array( 'downloadid' => $response->id ) );

		} 

	}

	/**
	 * It retrieves an asset's download link from the DASH API. If the link is ready, it  dynamically creates and saves a wordpress attachment into the media library. 
	 * 
	 * @since 	1.0.0
	 * @access 	public
	 * @return  array       A json payload containing the download link generated by the DASH API.
	 */
	public function dash_get_download() {

		if ( isset( 
			$_POST[ 'postdata' ], 
			$_POST[ 'postdata' ][ 'postid' ], 
			$_POST[ 'postdata' ][ 'imageid' ], 
			$_POST[ 'postdata' ][ 'imagefile' ],
			$_POST[ 'postdata' ][ 'downloadid' ], ) && 
			current_user_can( DASH_ROLE_CAPABILITY, intval( $_POST[ 'postdata' ][ 'postid' ] ) ) ) {

			$imagefile = sanitize_text_field( $_POST[ 'postdata' ][ 'imagefile' ] );
			$downloadid = sanitize_text_field( $_POST[ 'postdata' ][ 'downloadid' ] );
			$response = $this->api->getAssetDownload( $downloadid );

			$result = 'PENDING';
			$payload = array ( 'link' => $result );

			if ( isset( $response ) && $response->type === 'COMPLETED' ) {

				$url = $response->result->successes->{ 'download-1' }->result->url;

				// second, we upload the image into the media uploads directory
				$uploaddir = wp_upload_dir();
				$uploadfile = $uploaddir[ 'path' ] . '/' . $imagefile;

				$contents = file_get_contents( $url );
				$fhandle = fopen( $uploadfile, 'w' );
				fwrite( $fhandle, $contents );
				fclose( $fhandle );

				// third, update attachment details
				$wp_filetype = wp_check_filetype( basename( $imagefile ), null );
				$attachment = array(

				    'post_mime_type' => $wp_filetype[ 'type' ],
				    'post_title' => $imagefile,
				    'post_content' => '',
				    'post_status' => 'inherit'
				
				);

				$attach_id = wp_insert_attachment( $attachment, $uploadfile );
				$imagenew = get_post( $attach_id );
				$fullsizepath = get_attached_file( $imagenew->ID );
				$attach_data = wp_generate_attachment_metadata( $attach_id, $fullsizepath );

				wp_update_attachment_metadata( $attach_id, $attach_data ); 

				$result = wp_get_attachment_url( $attach_id );

				$payload = array( 'link' => $result, 'newid' => $imagenew->ID );

			}

			return wp_send_json_success( $payload );

		}

	}

	/**
	 * It retrieves the set of user defined transformation from the DASH api
	 * associated to a particular image.
	 *
	 * @since    1.0.0
	 * @param 	 string		$imageid the id of the image retrieved from the DASH API.
	 * @return   array      the set of preset trasformations in json format.
	 */
	public function dash_load_transforms( $imageid ) {

		if ( isset( $_POST[ 'postdata' ], $_POST[ 'postdata' ][ 'postid' ], $_POST[ 'postdata' ][ 'imageid' ] ) && 

			current_user_can( DASH_ROLE_CAPABILITY, intval( $_POST[ 'postdata' ][ 'postid' ] ) ) ) {

			$transforms = array();

			$imageid = sanitize_text_field( $_POST[ 'postdata' ][ 'imageid' ] );

			$response = $this->api->presetTransformationSearches( $imageid, 0, 100 );

			if ( isset( $response ) ) {

				$transforms = array_map( function( $e ) {

					$result = $e->result;

					return array(

						'id'   => $result->id,
						'name' => $result->name 

					);

				}, $response->results );

			}

			return wp_send_json_success( array( 'transforms' => $transforms ) );

		}
		
	}	

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Dash_Effortless_Digital_Asset_Management_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Dash_Effortless_Digital_Asset_Management_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/dash-effortless-digital-asset-management-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		$script_dependencies = array(
		    'dependencies' => array( 'jquery' ),
		    'version' => $this->version,
		);

		if ( file_exists( __DIR__ . '/js/build/dash-effortless-digital-asset-management-admin.asset.php' ) ) {

		    $script_dependencies = require __DIR__ . '/js/build/dash-effortless-digital-asset-management-admin.asset.php';

		}

		if ( current_user_can( DASH_ROLE_CAPABILITY ) )

			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/build/dash-effortless-digital-asset-management-admin.js', $script_dependencies[ 'dependencies' ], $script_dependencies[ 'version' ], false );

	}

	/**
	 * Register the media buttons action.
	 *
	 * @since 	 1.0.0
	 */
	public function media_buttons() {

		if ( current_user_can( DASH_ROLE_CAPABILITY ) )

			include( 'partials/dash-effortless-digital-asset-management-admin-button-display.php' );

	}

	/**
	 * It creates an appropriate preview URL for DASH images defaulting to 
	 * 324x324 thumbnails.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @param    string 	$url   	The size-less preview URL returned by DASH 
	 * @return 	 string             A preview URL that adds size data to the original URL.
	 */
	private function build_preview_url( $url ) {

		$parts = parse_url( $url );
		$defaultSizeStr = '/324x324.png?';

		$newUrl = $parts[ 'scheme' ] . '://' . 
				  $parts[ 'host' ] . 
				  $parts[ 'path' ] .
				  $defaultSizeStr  .
				  $parts[ 'query' ];

		return $newUrl;

	}

	/**
	 * Utility to extract attributes id information from an attributes payload 
	 * from the DASH API. If passed an attribute name, it checks if there is an entry
	 * for it in the API payload and it returns the attribute id or an empty string
	 * if no such property is found.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @param    string 	$attribute  	The name of an attribute to filter for
	 * @return 	 string                		A DASH id corresponding to the given attribute if found or an empty string otherwise.
	 */
	private function attribute_to_id( $attribute, $attribute_views ) {

		$obj = array_filter( $attribute_views, function( $item ) use ( $attribute ) {

			return !$item->hasFixedOptions && ( $item->name == $attribute ); 

		} );

		if ( count( $obj ) > 0 )

			return $obj[ 0 ]->id;

		return '';

	}

	/**
	 * It retrieves images from the user's DASH account and maps the DASH 
	 * payload to a wordpress media payload. If no $search parameter is specified
	 * this function performs a MATH_ALL image retrieval operation.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @param    string 	$search   A string of one or more keywords for keyword-based search.
	 * @return 	 array                An array of wordpress image objects.
	 */
	private function dash( $page, $page_size, $search ) {

		$dash_response = array();
		$images = array();

		// wordpress pages are 1-based but DASH's are 0-based
		$page = max( 0, $page - 1 );

		if ( $search !== NULL )
			$dash_response = $this->api->assetSearchByKeywords( $search, $page, $page_size );
		else
			$dash_response = $this->api->assetSearchAllImages( $page, $page_size );

		// get the attribute views
		$attribute_views = $this->api->attributeViews();

		foreach ( $dash_response->results as $obj ) {
			$image = $obj->result;
			$meta = $image->metadata;
			$file = $image->currentAssetFile;
			$orientation = strtolower( $file->orientation );

			$attr_title_id = $this->attribute_to_id( 'Title', $attribute_views );
			$attr_desc_id = $this->attribute_to_id( 'Description', $attribute_views );

			$title = $attr_title_id != '' ? $meta->values->{ $attr_title_id }[0] : '';
			$description = $attr_desc_id != '' ? $meta->values->{ $attr_desc_id }[0] : $title;

			$images[] = [
				"id" => hrtime( true ),
				"dashId" => $image->id,
				"isDash" => true,
				"title"=> $title,
				"filename"=> $file->filename,
				"url"=> $this->build_preview_url( $file->previewUrl ),
				"link"=> '',
				"alt"=> $title,
				"author"=> get_current_user_id(),
				"description"=>  $description,
				"caption"=>  '', 
				"name"=>  '', 
				"status"=> "inherit",
				"uploadedTo"=> 0,
				"date"=> $file->dateAdded,
				"modified"=> time(),
				"menuOrder"=> 0,
				"mime"=> $file->mediaType->type . '/' . $file->mediaType->subType,
				"type"=> 'image', 
				"subtype"=> $file->mediaType->subType,
				"icon"=> '' ,
				"dateFormatted"=> date("D M Y"),
				"nonces"=> [
					"update"=> "",
					"delete"=> "",
					"edit"=> ""
				],
				"editLink"=> "",
				"meta"=> false,
				"authorName"=> "",
				"uploadedToLink"=> "",
				"uploadedToTitle"=> "",
				"filesizeInBytes"=> 0,
				"filesizeHumanReadable"=> "Unknown",
				"context"=> "",
				"height"=> $file->dimensions->height,
				"width"=> $file->dimensions->width,
				"orientation"=> $orientation,
				"sizes" => array(),
				"compat"=> [
					"item"=> "",
					"meta"=> ""
				],
				"acf_errors"=> false
			];

		}

		return $images;

	}

	/**
	 * It populates the media modal when the query attachment event is triggered.
	 * 
	 * @since 	1.0.0
	 */
	public function ajax_query_attachments() {

		if ( isset( $_REQUEST[ 'query' ], $_REQUEST[ 'query' ][ 'post_mime_type' ] ) && $_REQUEST[ 'query' ][ 'post_mime_type' ] === 'dash-media' ) {

			// get pagination details
			$page = absint( $_REQUEST[ 'query' ][ 'paged' ] );
			$per_page = absint( $_REQUEST[ 'query' ][ 'posts_per_page' ] );

			// get search params/keywords
			$search = array_key_exists( 's', $_REQUEST[ 'query' ] ) ? sanitize_text_field( $_REQUEST[ 'query' ][ 's' ] ) : null;

			// get media from DASH
			$response = $this->dash( $page, $per_page, $search );

			return wp_send_json_success( $response );

		}

	}

	/**
	 * It displays an admin notice that reminds the admin to fill
	 * in the DASH API authentication settings.
	 * 
	 * @since 	1.0.0
	 */
	public function dash_admin_notice() {

		include( 'partials/dash-effortless-digital-asset-management-admin-notice-display.php' );

	}

	/**
	 * It loads a custom css for this plugin functionality into the TinyMCE iframe 
	 * if the editor is in classic editor mode.
	 * 
	 * @since 	1.0.0
	 * @param   array 		The array of settings for the TinyMCE iframe.
	 * @return  array       The modified settings.
	 * 
	 */
	public function mce_before_init( $settings ) {

		if ( array_key_exists( 'content_css', $settings ) )
			$settings[ 'content_css' ] = $settings[ 'content_css' ] . ',' . home_url() . '/app/plugins/dash-effortless-digital-asset-management/admin/css/dash-effortless-digital-asset-management-admin.css';

		return $settings;

	}

	/**
	 * It prints the template that dynamically adds image sizes into the 
	 * attachment details based on the image transforms retrieved from the DASH API.
	 * 
	 * @since 	1.0.0
	 */
	public function print_media_templates() {

		if ( ! isset( get_current_screen()->id ) || get_current_screen()->base != 'post' )
            return;

        ?>

        <script type="text/html" id="tmpl-dash-attachment-fields">
	        <# if ( data.hasOwnProperty( 'isDash' ) ) { 

	        	jQuery.post( ajaxurl, {

					'postdata': {

				 		'postid': jQuery("#post_ID").val(),
				 		'imageid': data.dashId,

				 	},

				 	'action': 'dash_load_transforms'

				 } ).done( function( response ) {

				 	const transforms = response.data.transforms;

				 	jQuery( '#dash-transforms-select' ).empty();

				 	jQuery.each( transforms, function( idx, transform ) {

				 		const selected = data.size ? ( data.size == transform.id ) : ( 'ORIGINAL' == transform.id );

				 		jQuery( '#dash-transforms-select' ).append(

				 			jQuery( '<option>', { value: transform.id , text: transform.name, selected: selected } )

				 		);

				 	} );

				 } );

	        #>
            <label class="setting">
                <span>Size</span>
                <select class="type" name="size" data-setting="size" id='dash-transforms-select'>
                </select>
            </label>
	        <# } #>
        </script>
        <?php

	}

}
