import { __ } from '@wordpress/i18n';

import React from 'react';

import { createHigherOrderComponent } from '@wordpress/compose'

import { Fragment } from '@wordpress/element'

import { Button, Spinner } from '@wordpress/components'

import { MediaPlaceholder, MediaUploadCheck, MediaUpload } from '@wordpress/block-editor';

import { addFilter } from '@wordpress/hooks'

import classnames from 'classnames';

const ALLOWED_MEDIA_TYPES = 'dash-media';

const DASH_MAX_POLL_RETRIES = 10;

const post = ( img, action, options ) => {

	const size = img.size || 'ORIGINAL';

	const postdata = {

		'postid': jQuery("#post_ID").val(),
		'imageid': img.dashId,
		'imagefile': img.filename,
		'imagetransf': size

	};

	const opts = options || {};

	return jQuery.post( ajaxurl, {

		'postdata': { ...postdata, ...opts },
		'action': action

	} );

}

const poll = ( img, downloadid, iter ) => {

	const _poll = ( iter, resolve, reject ) => {

		if ( iter >= DASH_MAX_POLL_RETRIES )
			return reject( new Error( 'Unable to get download link after max retries' ) );
		
		setTimeout( () => {

			post( img, 'dash_get_download', { downloadid: downloadid } )
			.then( ( result ) => {

				if ( result.data.link !== 'PENDING' ) 

					return resolve( result.data );

				return _poll( iter + 1, resolve, reject );

			} )
			.catch( () => {

				return reject();

			}, 200 );

		} );

	}
	
	return new Promise( ( resolve, reject ) => {

		_poll( iter, resolve, reject );

	} );

}

const setMediaWindow = () => {

	// hide the "Upload Files" tab
	jQuery( '.media-menu-item:contains("Upload Files")' ).css( 'display','none' );
	// ... in WP 5.8 ..
	jQuery( '#menu-item-upload.media-menu-item' ).css( 'display', 'none' );
	// hide the secondary filtering
	jQuery( '.media-toolbar-secondary' ).css( 'display', 'none' );
	// focus on the media browsing tab
	const mediatab = jQuery( '#menu-item-browse' ); 

	if ( mediatab.attr( 'aria-selected' ) != 'true' )
		mediatab.click();

}

( function( $ ) {

	function DASH() {

		var self = {
		
			collapseSelection: function() {}
		
		};

		var dashMediaWindowTitle = 'Insert Media from DASH';
		var spinner = null;

		var fn = {

			init: function() {

				// add button for the classic editor view
				$( document ).on( 'click', '.insert-dash-media-button', fn.open_media_window );

				// add a convenience handler for ajax errors
				$( document ).ajaxError( ( event, jqxhr, settings, thrownError ) => {

					const urlParams = new URLSearchParams( settings.data );
					const key = 'query[post_mime_type]';

					if ( urlParams.has( key ) && urlParams.get( key ) === 'dash-media' ) {

						if ( window.confirm( 'There was an error while performing this action. Please click OK to re-authenticate with DASH. If the error persists, please contact support.' ) ) {
							
							window.location.href='/wp/wp-admin/admin.php?page=dash_auth';
						
						};

					}

				} );

				// extend the backbone Attachment Details class with a reference to 
				// our template which add size fields dynamically
				// 
				// based on: https://gist.github.com/sunnyratilal/5650341
				//
				if( wp.media ) {

					wp.media.view.Attachment.Details = wp.media.view.Attachment.Details.extend( {
					    
					    initialize: function(){
					        // Always make sure that our content is up to date.
					        this.model.on( 'change', this.render, this );

					    },
					    render: function(){

					        // Ensure that the main attachment fields are rendered.
					        wp.media.view.Attachment.prototype.render.apply( this, arguments );

					        // Detach the views, append our custom fields, make sure that our data is fully updated and re-render the updated view.
					        this.views.detach();
					        this.$el.append( wp.media.template( 'dash-attachment-fields' )( this.model.toJSON() ) );
					        this.model.fetch();
					        this.views.render();

					        // This is the preferred convention for all render functions.
					        return this;
					    }

					} );

				}

			},

			open_media_window: function() {

				// we create a new media frame because even with the same
				// payload kapow may return us new recommendations
				self.window = wp.media( {

					title: dashMediaWindowTitle,
					library: { type: 'dash-media' },
					multiple: false,
					button: { text: 'Select' }

				} );

				self.window.on( 'select', function() {

					var img = self.window.state().get( 'selection' ).first().toJSON();

					wp.media.editor.insert( '<div class="lds-hourglass"></div>' );

					post( img, 'dash_download_image' )
					.then( ( response ) => {

						return poll( img, response.data.downloadid, 0 );

					} )
					.then( ( data ) => {

						img.id = data.newid;
						img.url = data.link;
						
						const ifr = $( '#content_ifr' );

						$( '.lds-hourglass', ifr.contents() ).remove();

						wp.media.editor.insert( '<img src="' + data.link + '" alt="' + img.alt + '">' );

					} )
					.catch( ( err ) => {

						console.error( err );

					} );

				} );

				self.window.on( 'ready', function () {

					setMediaWindow();
				
				} );

				self.window.open();
				return false;

			}

		}

		$( document ).ready( fn.init );

	}

	new DASH();

} )( jQuery );


const onOpen = ( open ) => {

	return () => {

		open( arguments );
		setMediaWindow();

	}

}

/**
 * Extend the MediaPlaceholder when in Gutenberg mode with the media button for DASH images.
 */
const mediaPlaceholderExtension = createHigherOrderComponent( MediaPlaceholder => {

	return ( props ) => {

		const [ image, setImage ] = React.useState( null );
		const [ waiting, setWaiting ] = React.useState( false );

		const isImgBlock = 'mediaPreview' in props;

		if( props.allowedTypes.includes( 'image' ) && isImgBlock ) {

			return (

				<Fragment>
					<MediaPlaceholder {...props}>
						{ waiting &&   

							<Spinner />

						}
						<MediaUploadCheck>
				 			<MediaUpload
								onSelect={ ( media ) => {

										setWaiting( true );

										post( media, 'dash_download_image' )
										.then( ( response ) => {

											return poll( media, response.data.downloadid, 0 ); 

										} )
										.then( ( data ) => {
											media.url = data.link;
											media.id = data.newid;
											setImage( media );
											setWaiting( false );
											props.onSelect( media );

										} )
										.catch( ( err ) => {

											console.error( err );

										} );

									}

								}
								title={ 'Insert Media from Dash' }
								allowedTypes={ ALLOWED_MEDIA_TYPES }
								value={ image ? image.id : '' }
								render={ ( { open } ) => (
									<Button 
										variant="tertiary" 
										className={ 
											isImgBlock ? 
												classnames( 
													'components-button', 
													'block-editor-media-placeholder__button', 
													'is-tertiary',
													'insert-guten-dash-media-button' 
												) :
												classnames( 
													'components-button', 
													'block-editor-media-placeholder__button', 
													'is-tertiary'
												)
										} 
										onClick={ onOpen( open ) }>{ __( 'Insert Media from Dash' ) }
									</Button>
								) }
							/>
						</MediaUploadCheck>
					</MediaPlaceholder>
				</Fragment>
		
			);

		}

		return <MediaPlaceholder {...props} />;

	};

}, 'mediaPlaceholderExtension' );

addFilter(

	'editor.MediaPlaceholder',
	'dash-effortless-dam/extend-media-placeholder',
	mediaPlaceholderExtension

);