<?php 

use Dash_Effortless_Digital_Asset_Management_OAuth as OAuth;
use Dash_Effortless_Digital_Asset_Management_Admin as Admin;
use Dash_Effortless_Digital_Asset_Management_Logging as Logger;

/**
 * DASH REST API wrapper
 *
 * @link       https://brightminded.com
 * @since      1.0.0
 *
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/includes
 */

/**
 * This class defines all functions required to interact with the DASH REST API.
 *
 * @since      1.0.0
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/includes
 * @author     Brightminded <brightminded@brightminded.com>
 */
class Dash_Effortless_Digital_Asset_Management_Api {

	/**
	 * It retrieves the authentication token to sent with every API request and
	 * refreshes it if it expires.
	 *
	 * @since 	 1.0.0
	 * @access   private
	 * @return	 string 		The bearer token to be used for DASH REST API requests.
	 */
	private function getAuthToken() {

		$tokenObj = get_option( DASH_AUTHENTICATED_TOKEN_KEY, NULL );

		if ( is_null( $tokenObj ) || empty( $tokenObj ) )
			throw new \Exception( 'Missing API Authentication Data' );

		$token = $tokenObj[ 'access' ];
		$expires = $tokenObj[ 'expires' ];
		$refresh = $tokenObj[ 'refresh' ];

		if ( $expires < time() ) {

			// token has expired...
			$settings = Admin::get_auth_settings();
			
			if ( isset( $refresh ) ) {

				//... attempt a refresh
				$tokenObj = OAuth::refresh(
					$settings[ DASH_OAUTH_DB_CLIENT_ID_KEY ],
	        		$settings[ DASH_OAUTH_DB_CLIENT_SECRET_KEY ],
	        		$settings[ DASH_OAUTH_REDIRECT_URI_KEY ],
	        		$refresh
				);

				if ( $tokenObj ) {

					Admin::set_auth_token( $tokenObj );
					$token = $tokenObj->getToken();

				}

			}
			else {

				//...otherwise throw an exception which will result
				// in an ajax error on the caller where the handler
				// can show a javascript message to the user.
				throw new Exception( 'Unauthorized' );

			}

		}

		return $token;

	}

	/**
	 * Makes an HTTP(s) request to the DASH API.
	 *
	 * @since 	 1.0.0
	 * @access   private
	 * @param    string $method		The HTTP request method e.g. GET, POST.
	 * @param 	 string $route 		Path relative to the DASH API base url for the required REST resource.
	 * @param    array $data        The payload to send to the DASH REST API.
	 * @return	 array 				Any response from the DASH REST API.
	 */
	private function request( $method, $route, $data ) {

		$resource = DASH_API_BASE_URL . $route;

		$params = array( 

			'method'		=> $method,
			'headers'		=> array( 

				'Authorization'	=> 'Bearer ' . $this->getAuthToken(),
				'Content-Type'	=> 'application/json'

			),
			'timeout'		=> 45,
			'redirection'	=> 5,
			'blocking'		=> true,
			'httpversion'	=> '1.1'

		);

		if ( $method === 'POST' && $data )
			$params[ 'body' ] = wp_json_encode( $data );

		$response = wp_remote_request( $resource, $params );

		if ( $response[ 'response' ][ 'code' ] > 200 )
			Logger::error( get_class(), 'API response status = ' . $response[ 'response' ][ 'code' ] );

		return json_decode( wp_remote_retrieve_body( $response ) );

	}

	/**
	 * Makes an HTTP(s) POST request to the DASH API.
	 *
	 * @since 	 1.0.0
	 * @access   private
	 * @param 	 string $route 		Path relative to the DASH API base url for the required REST resource.
	 * @param    array $payload     The payload to send to the DASH REST API.
	 * @return	 array 				Any response from the DASH REST API.
	 */
	private function post( $route, $payload ) {

		$results = $this->request( 

			'POST',
			$route,
			$payload

		);

		return $results ? $results : array();

	}


	/**
	 * Makes a HTTP(s) GET request to the DASH API.
	 *
	 * @since 	 1.0.0
	 * @access   private
	 * @param 	 string $route 		Path relative to the DASH API base url for the required REST resource.
	 * @return	 array 				Any response from the DASH REST API.
	 */
	private function get( $route ) {

		$results = $this->request( 

			'GET',
			$route,
			NULL

		);

		return $results ? $results : array();

	}

	/**
	 * It requests a MATCH_ALL asset search from the DASH API.
	 *
	 * @reference: https://brightdash.redoc.ly/#operation/postAssetSearch
	 * 
	 * @since 	 1.0.0
	 * @access   public
	 * @param    integer $from		The item number to begin the result set from.
	 * @param 	 integer $pageSize	The maximum number of items to return in the result setREST resource.
	 * @return	 array 				List of search results or an empty array.
	 */
	public function assetSearchMatchAll( $from, $pageSize ) {

		$route = '/asset-searches';

		$payload = array(

			'from'		=> $from,
			'pageSize'	=> $pageSize,
			'criterion'	=> array( 'type' => 'MATCH_ALL' ), 
			'sorts'		=> array()	

		);

		return $this->post( $route, $payload );

	}

	/**
	 * It requests a search for all assets of type Image from the DASH API.
	 * 
	 * @reference: https://brightdash.redoc.ly/#operation/postAssetSearch
	 *
	 * @since 	 1.0.0
	 * @access   public
	 * @param    integer $from		The item number to begin the result set from.
	 * @param 	 integer $pageSize	The maximum number of items to return in the result setREST resource.
	 * @return	 array 				List of search results or an empty array.
	 */
	public function assetSearchAllImages( $from, $pageSize ) {

		$route = '/asset-searches';

		$payload = array(

			'from' 		=> $from,
			'pageSize'	=> $pageSize,
			'criterion'	=> array(

				'type'	=> 'FIELD_MATCHES',
				'value' => 'Image',
				'field'	=> array(

					'type'		=> 'FIXED',
					'fieldName'	=> 'FILE_TYPE'

				)

			),
			'sorts' 	=> array(

				array(

					'field' => array(

						'type' 		=> 'FIXED',
						'fieldName'	=> 'DATE_ADDED'

					),
					'order'	=> 'DESC'

				)

			)

		);

		return $this->post( $route, $payload );

	}

	/**
	 * It requests a KEYWORDS asset search from the DASH API where the 
	 * result is constrained to be of type Image only.
	 * 
	 * @reference: https://brightdash.redoc.ly/#operation/postAssetSearch
	 *
	 * @since 	 1.0.0
	 * @access   public
	 * @param    array   $keywords  The list of keywords to search against.
	 * @param    integer $from		The item number to begin the result set from.
	 * @param 	 integer $pageSize	The maximum number of items to return in the result setREST resource.
	 * @return	 array 				List of search results or an empty array.
	 */
	public function assetSearchByKeywords( $keywords, $from, $pageSize ) {

		$route = '/asset-searches';

		$payload = array(

			'from' 		=> $from,
			'pageSize'	=> $pageSize,
			'criterion'	=> array(
				'type'	   => 'AND',
				'criteria' => array(

					array(
						'type'	=> 'FIELD_MATCHES',
						'value' => 'Image',
						'field'	=> array(

							'type'		=> 'FIXED',
							'fieldName'	=> 'FILE_TYPE'

						)
					),

					array(
						'type'	=> 'FIELD_MATCHES',
						'value' => $keywords,
						'field'	=> array(

							'type'		=> 'FIXED',
							'fieldName'	=> 'KEYWORDS'

						)	

					)

				)

			),
			'sorts' 	=> array()

		);

		return $this->post( $route, $payload );

	}

	/**
	 * Used for finding presets that are applicable to a set of assets, so that one can be selected for use to create some Asset Downloads from the DASH API.
	 * 
	 * @reference: https://brightdash.redoc.ly/#tag/Preset-Transformations
	 *
	 * @since 	 1.0.0
	 * @access   public
	 * @param    string  $assetId   The id of the asset to get transforms for.
	 * @param    integer $from		The item number to begin the result set from.
	 * @param 	 integer $pageSize	The maximum number of items to return in the result setREST resource.
	 * @return	 array 				List of search results or an empty array.
	 */
	public function presetTransformationSearches( $assetId, $from, $pageSize ) {

		$route = '/preset-transformation-searches';

		$payload = array(

			'from'		=> $from,
			'pageSize'	=> $pageSize,
			'criterion'	=> array(

				'type'		=> 'AND',
				'criteria'  => array(

					array(

						'type' 		=> 'APPLICABLE_FOR_ANY_ASSETS',
						'assetIds' 	=> array( $assetId )

					),

					array(

						'type' 		=> 'NOT',
						'criterion'	=> array(

							'type'	=> 'HAS_TRANSFORMATION_OF_TYPE',
							'transformationType' => 'CROP'

						)

					)

				)

			),
			'sorts'		=> array()

		);

		return $this->post( $route, $payload );

	}

	/**
	 * It requests an attribute resource from the DASH API.
	 * 
	 * @reference: https://brightdash.redoc.ly/#tag/Attributes
	 *
	 * @since 	 1.0.0
	 * @access   public
	 * @param    string  $attributeId  The id of the attribute resource to retrieve.
	 * @return	 array 				   Attribute resource data.
	 */
	public function getAttribute( $attributeId ) {

		$route = '/attributes/' . $attributeId;

		return $this->get( $route );

	}

	/**
	 * It requests attribute views from the DASH API.
	 * 
	 * @reference: https://brightdash.redoc.ly/#tag/Attribute-Views
	 *
	 * @since 	 1.0.0
	 * @access   public
	 * @param    string  $attributeId  The id of the attribute resource to retrieve.
	 * @return	 array 				   Attribute resource data.
	 */
	public function attributeViews() {

		$route = '/attribute-views';

		// only return the properties with hasFixedOption=false from 
		// the INTERNAL_SUMMARY results
		$result = $this->get( $route );

		$internal_summary = array_filter( $result, function( $item ) {

			return $item->result->type == 'INTERNAL_SUMMARY';

		} );

		if ( count( $internal_summary ) > 0 )
			return $internal_summary[ 0 ]->result->attributes;

		return array(); 

	}

	/**
	 * It requests an asset download with optional transform from the DASH API.
	 * 
	 * @reference: https://brightdash.redoc.ly/#operation/postAssetDownloadBatchJob
	 *
	 * @since 	 1.0.0
	 * @access   public
	 * @param    string  $assetId  	   The id of the asset to prepare for download.
	 * @return	 array 				   Download progress payload.
	 */
	public function assetDownload( $assetId, $transformationId ) {

		$route = '/asset-download-batch-jobs';

		$payload = array(

			'items'		=> array(

				array(

					'batchItemId'	=> 'download-1',
					'assetId'		=> $assetId

				)

			),

			'zip'		=> false

		);


		if ( isset( $transformationId ) ) {

			$payload[ 'transformationDescription' ] = array(

				'type'		=> 'PRESET',
				'presetTransformationId'	=> $transformationId

			);
			
		}

		return $this->post( $route, $payload );

	}

	/**
	 * It requests the status and eventual result of an asset download from the DASH API.
	 * 
	 * @reference: https://brightdash.redoc.ly/#operation/getAssetDownloadBatchJob
	 *
	 * @since 	 1.0.0
	 * @access   public
	 * @param    string  $assetId  	   The id of the asset to prepare for download.
	 * @return	 array 				   Download progress payload.
	 */
	public function getAssetDownload( $downloadId ) {

		$route = '/asset-download-batch-jobs/' . $downloadId;

		return $this->get( $route );

	}

}