<?php 

use \League\OAuth2\Client\Provider\GenericProvider;
use \League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use Dash_Effortless_Digital_Asset_Management_Logging as Logger;

/**
 * DASH REST OAuth2 Authentication wrapper
 *
 * @link       https://brightminded.com
 * @since      1.0.0
 *
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/includes
 */

/**
 * This class defines all functions required to perform OAuth2 authentication with the DASH REST API.
 * 
 * @reference: https://brightdash.redoc.ly/#section/Obtaining-a-Bearer-Token
 *
 * @since      1.0.0
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/includes
 * @author     Brightminded <brightminded@brightminded.com>
 */
class Dash_Effortless_Digital_Asset_Management_OAuth {

    /**
     * It creates a new OAuth authorization framework provider.
     * 
     * @reference: https://oauth2-client.thephpleague.com/
     *
     * @since    1.0.0
     * @access   private
     * @param    string  $assetId      The id of the asset to prepare for download.
     * @return   array                 Download progress payload.
     */
    private static function getProvider( $clientId, $clientSecret, $redirectUri ) {

        return new GenericProvider(

            array(

                'clientId'                => $clientId,
                'clientSecret'            => $clientSecret,
                'redirectUri'             => $redirectUri,
                'urlAuthorize'            => DASH_LOGIN_BASE_URL . '/authorize',
                'urlAccessToken'          => DASH_LOGIN_BASE_URL . '/oauth/token',
                'urlResourceOwnerDetails' => '' 

            )

        );

    }

    /**
     * It performs the OAuth authorization step whereby permission is requested
     * for the plugin to make DASH API requests on behalf of the user.
     * 
     * @reference https://auth0.com/docs/authenticate/protocols/oauth
     *
     * @since    1.0.0
     * @access   public
     * @param    string  $clientId     The DASH clientId identifying the user.
     * @param    string  $clientSecret The DASH clientSecret identifying the user.
     * @param    string  $redirectUri  The local URI containing the OAuth callback functionality.
     * @param    string  $subdomain    The DASH subdomain identifying the user.
     * 
     * @return   This function redirects the user to a DASH login form. Successful login will authorize this plugin to make DASH REST API requests on their behalf.
     */
    public static function authorize( $clientId, $clientSecret, $redirectUri, $subdomain ) {

        $provider = self::getProvider( $clientId, $clientSecret, $redirectUri );

        $options = array(

            'response_type' => 'code',
            'client_secret' => $clientSecret,
            'audience'      => DASH_OAUTH_AUDIENCE,
            'scope'         => DASH_OAUTH_DEFAULT_SCOPE . ' subdomain:' . $subdomain

        );

        $authorizationUrl = $provider->getAuthorizationUrl( $options );

        $_SESSION[ 'oauth2state' ] = $provider->getState();

        header( 'Location: ' . $authorizationUrl );

        exit;

    }

    /**
     * It performs the OAuth token request step whereby an authentication 
     * token is requested that will be used to in request headers to
     * authenticate DASH API requests.
     * 
     * @reference: https://auth0.com/docs/authenticate/protocols/oauth
     *
     * @since    1.0.0
     * @access   public
     * @param    string  $clientId          The DASH clientId identifying the user.
     * @param    string  $clientSecret      The DASH clientSecret identifying the user.
     * @param    string  $redirectUri       The local URI containing the OAuth callback functionality.
     * @param    string  $authorizationCode The code that was returned by the DASH API during the authorize step.
     * 
     * @return   League\OAuth2\Client\Token\AccessTokenInterface
     */
    public static function token( $clientId, $clientSecret, $redirectUri, $authorizationCode ) {

        return self::getToken( 
        
            $clientId, 
            $clientSecret, 
            $redirectUri, 
            'authorization_code', 
            array( 'code' => $authorizationCode )
        
        );

    }

    /**
     * It performs the OAuth token refresh step whereby an authentication 
     * token is refreshed that will be used to in request headers to
     * authenticate DASH API requests.
     * 
     * @reference: https://auth0.com/docs/authenticate/protocols/oauth
     *
     * @since    1.0.0
     * @access   public
     * @param    string  $clientId          The DASH clientId identifying the user.
     * @param    string  $clientSecret      The DASH clientSecret identifying the user.
     * @param    string  $redirectUri       The local URI containing the OAuth callback functionality.
     * @param    string  $refreshToken      The refresh token obtained from the DASH API.
     * 
     * @return   League\OAuth2\Client\Token\AccessTokenInterface
     */
    public static function refresh( $clientId, $clientSecret, $redirectUri, $refreshToken ) {

        return self::getToken(

            $clientId,
            $clientSecret,
            $redirectUri,
            'refresh_token',
            array( 'refresh_token' => $refreshToken )
        
        );

    }

    /**
     * It performs the OAuth request for eiter a new token or for
     * a token refresh.
     * 
     * @reference: https://auth0.com/docs/authenticate/protocols/oauth
     *
     * @since    1.0.0
     * @access   private
     * @param    string  $clientId          The DASH clientId identifying the user.
     * @param    string  $clientSecret      The DASH clientSecret identifying the user.
     * @param    string  $redirectUri       The local URI containing the OAuth callback functionality.
     * @param    string  $type              Either authorization_code or refresh_token.
     * @param    array   $payload           Either the authorization code or the refresh token
     * 
     * @return   League\OAuth2\Client\Token\AccessTokenInterface
     */
    private static function getToken( $clientId, $clientSecret, $redirectUri, $type, $payload ) {

        $provider = self::getProvider( $clientId, $clientSecret, $redirectUri );
        
        $token = null;

        try {

            $token = $provider->getAccessToken(

                $type,
                $payload

            );

        }
        catch( IdentityProviderException $e ) {

            Logger::error( get_class(), $e->getMessage() );

        }
        finally {

            return $token;

        }

    }

}