<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://brightminded.com
 * @since      1.0.0
 *
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Dash_Effortless_Digital_Asset_Management
 * @subpackage Dash_Effortless_Digital_Asset_Management/includes
 * @author     Brightminded <brightminded@brightminded.com>
 */
class Dash_Effortless_Digital_Asset_Management {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Dash_Effortless_Digital_Asset_Management_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		if ( defined( 'DASH_EFFORTLESS_DIGITAL_ASSET_MANAGEMENT_VERSION' ) ) {
			$this->version = DASH_EFFORTLESS_DIGITAL_ASSET_MANAGEMENT_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'dash-effortless-digital-asset-management';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Dash_Effortless_Digital_Asset_Management_Loader. Orchestrates the hooks of the plugin.
	 * - Dash_Effortless_Digital_Asset_Management_i18n. Defines internationalization functionality.
	 * - Dash_Effortless_Digital_Asset_Management_Admin. Defines all hooks for the admin area.
	 * - Dash_Effortless_Digital_Asset_Management_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-dash-effortless-digital-asset-management-loader.php';

		/**
		 * The autoloader for the external vendor dependencies
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';

		/**
		 * The class responsible for defining internationalization functionality of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-dash-effortless-digital-asset-management-i18n.php';

		/**
		 * The wrapper class for the DASH REST API.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-dash-effortless-digital-asset-management-api.php';

		/**
		 * The wrapper class for OAuth2 authentication with the DASH REST API.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-dash-effortless-digital-asset-management-oauth.php';

		/**
		 * The wrapper class for simple Logging.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-dash-effortless-digital-asset-management-logging.php';
		
		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-dash-effortless-digital-asset-management-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-dash-effortless-digital-asset-management-public.php';
		

		$this->loader = new Dash_Effortless_Digital_Asset_Management_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Dash_Effortless_Digital_Asset_Management_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Dash_Effortless_Digital_Asset_Management_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Dash_Effortless_Digital_Asset_Management_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_menu', $plugin_admin, 'dash_admin_page' );
		$this->loader->add_action( 'print_media_templates', $plugin_admin, 'print_media_templates' );
		$this->loader->add_filter( 'tiny_mce_before_init', $plugin_admin, 'mce_before_init' );
		$this->loader->add_action( 'admin_post_dash_auth', $plugin_admin, 'dash_auth' );
		$this->loader->add_action( 'current_screen', $plugin_admin, 'dash_auth_callback' );
		$this->loader->add_action( 'wp_ajax_oauth_token', $plugin_admin, 'dash_token' );
		$this->loader->add_action( 'media_buttons', $plugin_admin, 'media_buttons', 15 );
		$this->loader->add_action( 'wp_ajax_query-attachments', $plugin_admin, 'ajax_query_attachments', -1 );
		$this->loader->add_action( 'wp_ajax_dash_download_image', $plugin_admin, 'dash_download_image' );
		$this->loader->add_action( 'wp_ajax_dash_get_download', $plugin_admin, 'dash_get_download' );
		$this->loader->add_action( 'wp_ajax_dash_load_transforms', $plugin_admin, 'dash_load_transforms' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// remind the user to enter the DASH details
		$tokenObj = get_option( DASH_AUTHENTICATED_TOKEN_KEY, NULL );

		if ( is_null( $tokenObj ) || empty( $tokenObj ) ) 
			$this->loader->add_action( 'admin_notices', $plugin_admin, 'dash_admin_notice' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Dash_Effortless_Digital_Asset_Management_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Dash_Effortless_Digital_Asset_Management_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
